## $Id: browser.py 13066 2015-06-16 14:48:55Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""UI components for products offered by the company.
"""

import sys
import grok
import pytz
from urllib import urlencode
from datetime import datetime
from zope.event import notify
from zope.i18n import translate
from zope.catalog.interfaces import ICatalog
from zope.component import queryUtility, getUtility, createObject
from zope.schema.interfaces import ConstraintNotSatisfied, RequiredMissing
from zope.formlib.textwidgets import BytesDisplayWidget
from zope.security import checkPermission
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.interfaces import (
    IContactForm, IObjectHistory, IIkobaObject, IIkobaUtils)
from waeup.ikoba.browser.layout import (
    IkobaPage, IkobaEditFormPage, IkobaAddFormPage, IkobaDisplayFormPage,
    NullValidator, jsaction, action, UtilityView)
from waeup.ikoba.widgets.datewidget import (
    FriendlyDateWidget, FriendlyDateDisplayWidget,
    FriendlyDatetimeDisplayWidget)
from waeup.ikoba.browser.breadcrumbs import Breadcrumb
from waeup.ikoba.utils.helpers import html2dict
from waeup.ikoba.browser.pages import (
    delSubobjects, add_local_role, del_local_roles, msave,
    LocalRoleAssignmentUtilityView)

from waeup.ikoba.products.interfaces import (
    IProductsContainer, IProduct)

grok.context(IIkobaObject) # Make IKofaObject the default context
grok.templatedir('browser_templates')


class ProductsBreadcrumb(Breadcrumb):
    """A breadcrumb for the customers container.
    """
    grok.context(IProductsContainer)
    title = _('Products')


class ProductBreadcrumb(Breadcrumb):
    """A breadcrumb for the customer container.
    """
    grok.context(IProduct)

    def title(self):
        return self.context.title


class ProductsContainerPage(IkobaDisplayFormPage):
    """The standard view for product containers.
    """
    grok.context(IProductsContainer)
    grok.name('index')
    grok.require('waeup.Public')
    grok.template('containerpage')
    pnav = 1
    label = _('Products')

    @property
    def description(self):
        lang = self.request.cookies.get('ikoba.language')
        html = self.context.description_dict.get(lang,'')
        if html =='':
            portal_language = getUtility(IIkobaUtils).PORTAL_LANGUAGE
            html = self.context.description_dict.get(portal_language,'')
        return html


class ProductsContainerManageFormPage(IkobaEditFormPage,
                                      LocalRoleAssignmentUtilityView):
    """The manage page for customer containers.
    """
    grok.context(IProductsContainer)
    grok.name('manage')
    grok.require('waeup.manageProducts')
    grok.template('containermanagepage')
    pnav = 1
    label = _('Manage product section')

    taboneactions = [_('Save'), _('Cancel')]
    tabtwoactions = [_('Add product'), _('Remove selected'), _('Cancel')]

    @action(_('Add product'), style='primary', validator=NullValidator)
    def addSubunit(self, **data):
        self.redirect(self.url(self.context, 'addproduct'))
        return

    @jsaction(_('Remove selected'))
    def delProducts(self, **data):
        delSubobjects(self, redirect='manage', tab='2')
        return

    @action(_('Save'), style='primary')
    def save(self, **data):
        msave(self, **data)
        # Save multilingual dict
        portal_language = getUtility(IIkobaUtils).PORTAL_LANGUAGE
        desc = getattr(self.context, 'description', None)
        self.context.description_dict = html2dict(desc, portal_language)
        return 

    @action(_('Cancel'), validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return


class ProductAddFormPage(IkobaAddFormPage):
    """Add-form to add a customer.
    """
    grok.context(IProductsContainer)
    grok.require('waeup.manageProducts')
    grok.name('addproduct')
    form_fields = grok.AutoFields(IProduct)
    label = _('Add product')
    pnav = 1

    @action(_('Create product'), style='primary')
    def addProduct(self, **data):
        product = createObject(u'waeup.Product')
        self.applyData(product, **data)
        try:
            self.context.addProduct(product)
        except KeyError:
            self.flash(_('The id chosen already exists.'),
                       type='danger')
            return
        self.flash(
            _("Product ${a} added.", mapping = {'a':data['product_id']}))
        ob_class = self.__implemented__.__name__.replace('waeup.ikoba.','')
        self.context.__parent__.logger.info(
            '%s - added: %s' % (ob_class, data['product_id']))
        self.redirect(self.url(self.context, u'manage')+'#tab2')
        return


class ProductDisplayFormPage(IkobaDisplayFormPage):
    """ Page to display product data
    """
    grok.context(IProduct)
    grok.name('index')
    grok.require('waeup.Public')
    grok.template('productpage')
    #grok.template('basepage')
    pnav = 1

    @property
    def form_fields(self):
        return grok.AutoFields(self.context.form_fields_interface).omit(
            'terms_and_conditions', 'description')

    @property
    def label(self):
        return self.context.title

    @property
    def description(self):
        lang = self.request.cookies.get('ikoba.language')
        html = self.context.description_dict.get(lang,'')
        if html =='':
            portal_language = getUtility(IIkobaUtils).PORTAL_LANGUAGE
            html = self.context.description_dict.get(portal_language,'')
        return html

    @property
    def terms_and_conditions(self):
        lang = self.request.cookies.get('ikoba.language')
        html = self.context.tc_dict.get(lang,'')
        if html =='':
            portal_language = getUtility(IIkobaUtils).PORTAL_LANGUAGE
            html = self.context.tc_dict.get(portal_language,'')
        return html


class ProductManageFormPage(IkobaEditFormPage,
                            LocalRoleAssignmentUtilityView):
    """ View to manage product data
    """
    grok.context(IProduct)
    grok.name('manage')
    grok.require('waeup.manageProducts')
    grok.template('productmanagepage')
    pnav = 1

    taboneactions = [_('Save'), _('Cancel')]
    tabtwoactions1 = [_('Remove selected local roles')]
    tabtwoactions2 = [_('Add local role')]


    @property
    def form_fields(self):
        return grok.AutoFields(
            self.context.form_fields_interface).omit('product_id')

    def label(self):
        return _("Manage product ${a}", mapping = {'a':self.context.product_id})

    @action(_('Save'), style='primary')
    def save(self, **data):
        msave(self, **data)
        # Save multilingual dicts
        portal_language = getUtility(IIkobaUtils).PORTAL_LANGUAGE
        desc = getattr(self.context, 'description', None)
        self.context.description_dict = html2dict(desc, portal_language)
        tc = getattr(self.context, 'terms_and_conditions', None)
        self.context.tc_dict = html2dict(tc, portal_language)
        return 

    @action(_('Cancel'), validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

    @action(_('Add local role'), validator=NullValidator)
    def addLocalRole(self, **data):
        return add_local_role(self,2,**data)

    @action(_('Remove selected local roles'))
    def delLocalRoles(self, **data):
        return del_local_roles(self,2,**data)